/* Copyright (C) 2007 Andre Noll <maan@tuebingen.mpg.de>, see file COPYING. */

/** \file signal.h exported symbols from signal.c */

/**
 * Task for signal handling.
 */
struct signal_task {
	/** The signal pipe. */
	int fd;
	/** The associated task structure. */
	struct task *task;
};

/**
 * A generic pre-select method for signal tasks.
 *
 * \param s Passed to para_fd_set().
 * \param context Signal task pointer.
 *
 * This convenience helper is called from several programs which need to handle
 * signals, including para_server and para_audiod. These programs define a
 * signal task structure and set its ->pre_select method to this function which
 * adds the file descriptor of the signal task to the set of descriptors to be
 * watched in the next select() call.
 *
 * Although the second parameter must be in fact a pointer to a signal_task
 * structure, the parameter is specified as void * here to match the
 * ->pre_select method of struct task.
 */
_static_inline_ void signal_pre_select(struct sched *s, void *context)
{
	struct signal_task *st = context;
	para_fd_set(st->fd, &s->rfds, &s->max_fileno);
}

struct signal_task *signal_init_or_die(void);
void para_sigaction(int sig, void (*handler)(int));
void para_install_sighandler(int);
int para_reap_child(pid_t *pid);
int para_next_signal(fd_set *rfds);
void signal_shutdown(struct signal_task *st);
void para_block_signal(int sig);
void para_unblock_signal(int sig);
